#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# vim: fenc=utf-8
# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4
#
#

"""
File name: cli.py
Author: dhilipsiva <dhilipsiva@gmail.com>
Date created: 2016-06-05
"""

import click
from json import dumps, loads
from hashlib import md5
from getpass import getuser
from Crypto.Cipher import AES
from os.path import expanduser, isfile
from pwdman.core import PwdMan

VERSION = "1"
SETUP_MESSAGE = """
Setup Done!
Please add these dollowing lines in your `.bashrc` or `.bash_profile`

export PWDMAN_SALT=%(salt)s
expoer PWDMAN_PATH=%(path)s

Be sure to open a new terminal - to load new ENV variables.

Example: To generate password for GitHub, Type:

`pwdman github`

"""


def save_config(salt, path, config):
    salt_md5 = md5(salt.encode()).hexdigest()
    obj = AES.new(
        salt_md5, AES.MODE_CTR, counter=lambda: salt_md5[:16].encode())
    message = dumps(config)
    ciphertext = obj.encrypt(message)
    with open(path, 'wb') as config_file:
        config_file.write(ciphertext)
    return ciphertext


def read_config(salt, path):
    salt_md5 = md5(salt.encode()).hexdigest()
    obj = AES.new(
        salt_md5, AES.MODE_CTR, counter=lambda: salt_md5[:16].encode())
    with open(path, 'rb') as config_file:
        data = obj.decrypt(config_file.read())
    return loads(data.decode())


def get_kwargs(config, service, length, numbers, lowers, uppers, symbols):
    s = config["services"].get(service)
    passphrase = config["passphrase"]
    if s is None:
        if length is None:
            length = 25
        if numbers is None:
            numbers = True
        if lowers is None:
            lowers = True
        if uppers is None:
            uppers = True
        if symbols is None:
            symbols = True
    else:
        if length is None:
            length = s["length"]
        if numbers is None:
            numbers = s["numbers"]
        if lowers is None:
            lowers = s["lowers"]
        if uppers is None:
            uppers = s["uppers"]
        if symbols is None:
            symbols = s["symbols"]

    return dict(
        passphrase=passphrase, length=length, numbers=numbers, lowers=lowers,
        uppers=uppers, symbols=symbols)


@click.command()
@click.argument('service')
@click.option('--salt', envvar='PWDMAN_SALT', type=str, default=getuser)
@click.option('--path', envvar='PWDMAN_PATH', type=str, default="~/pwdman")
@click.option('--length', type=int, default=None)
@click.option('--numbers/--no-numbers', default=None)
@click.option('--lowers/--no-lowers', default=None)
@click.option('--uppers/--no-uppers', default=None)
@click.option('--symbols/--no-symbols', default=None)
def cli(service, salt, path, length, numbers, lowers, uppers, symbols):
    path = expanduser(path)
    config = read_config(salt, path)
    kwargs = get_kwargs(
        config, service, length, numbers, lowers, uppers, symbols)
    pm = PwdMan(**kwargs)
    print(pm.generate_password(service))
    config["services"][service] = kwargs
    save_config(salt, path, config)


@click.command()
@click.option('--salt', envvar='PWDMAN_SALT', type=str, default=getuser)
@click.option('--path', envvar='PWDMAN_PATH', type=str, default="~/pwdman")
@click.option('--passphrase', type=str, prompt=True)
def setup(salt, path, passphrase):
    path = expanduser(path)
    if isfile(path):
        print("Setup Already Done!")
    else:
        config = dict(passphrase=passphrase, services={}, version=VERSION)
        save_config(salt, path, config)
        print(SETUP_MESSAGE % locals())



#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# vim: fenc=utf-8
# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4
#
#

"""
File name: core.py
Author: dhilipsiva <dhilipsiva@gmail.com>
Date created: 2016-06-04
"""

from hashlib import md5
from itertools import cycle

NUMBERS = list("0123456789")
LETTERS_LOWER = list("abcdefghijklmnopqrstuvwxyz")
LETTERS_UPPER = list("ABCDEFGHIJKLMNOPQRSTUVWXYZ")
SYMBOLS = list("~@#^&*()_+{|}:<>?-[];,.")


def _get_numerical_digest(text):
    text = text.encode()
    hex_digest = md5(text).hexdigest()
    list_digest = list(hex_digest)

    numerical_digest = []
    for element in list_digest:
        numerical_digest.append(int(element, 16))
    return numerical_digest


class PwdMan(object):
    def __init__(
            self, passphrase, length=25, numbers=True, lowers=True,
            uppers=True, symbols=True):

        self.true_count = list(locals().values()).count(True)
        self.min_count = int(length/self.true_count)
        self.allowed = []
        self.pwd = ""
        self.passphrase = passphrase
        self.length = length
        self.numbers = numbers
        self.lowers = lowers
        self.uppers = uppers
        self.symbols = symbols

    def password_segment(self, allowed, length=None):
        self.allowed += allowed
        length = length or self.min_count
        text = self.passphrase + self.service + str(length) + "".join(allowed)
        numerical_digest = _get_numerical_digest(text)
        cycler_digest = cycle(numerical_digest)
        cycler_allowed = cycle(allowed)
        segment = ""
        for l in range(length):
            letter = next(cycler_allowed)
            num = next(cycler_digest)
            for n in range(num):
                letter = next(cycler_allowed)
            segment += letter

        self.pwd += segment

    def shuffle_password(self):
        new_pwd = []
        old_pwd = list(self.pwd)
        numerical_digest = _get_numerical_digest(self.pwd)
        cycler_numerical = cycle(numerical_digest)
        for l in range(self.length):
            num = next(cycler_numerical)
            cycler_digest = cycle(old_pwd)
            letter = next(cycler_digest)
            for n in range(num):
                letter = next(cycler_digest)
            new_pwd.append(old_pwd.pop(old_pwd.index(letter)))
        self.pwd = "".join(new_pwd)

    def generate_password(self, service):
        self.pwd = ""
        self.service = service

        # Filters
        if self.numbers:
            self.password_segment(NUMBERS)
        if self.lowers:
            self.password_segment(LETTERS_LOWER)
        if self.uppers:
            self.password_segment(LETTERS_UPPER)
        if self.symbols:
            self.password_segment(SYMBOLS)

        count_remaining = self.length - (self.min_count * self.true_count)
        self.password_segment(self.allowed, count_remaining)

        self.shuffle_password()

        return self.pwd



#! /usr/bin/env python
# -*- coding: utf-8 -*-
#
# vim: fenc=utf-8
# vim: tabstop=4 expandtab shiftwidth=4 softtabstop=4
#
#

"""
File name: setup.py
Author: dhilipsiva <dhilipsiva@gmail.com>
Date created: 2016-06-05
"""

from setuptools import setup, find_packages

setup(
    name='pwdman',
    version='0.2',
    packages=find_packages(),
    include_package_data=True,
    install_requires=[
        'Click',
        'pycrypto',
    ],
    entry_points='''
        [console_scripts]
        pwdman=pwdman.cli:cli
        pwdman-setup=pwdman.cli:setup
    ''',
)
